
"""

Kibrit
======

The module for watching SCM (git, hg, svn).
Use it for control a static's versions.

"""

__version__ = '0.1.5'
__project__ = __name__
__author__ = "Sinan Midillili <sinan@hipo.biz>"
__license__ = "BSD"


from os import name
import os
from subprocess import Popen, PIPE
import sys

from django.core.cache import cache

from django_kibrit.constants import DJANGO_KIBRIT_CACHED_KEY_ID


class GitRevision(object):
    """
    Return the 8 character long git revision number or an empty string
    """

    def __init__(self, path=None):
        """
        Get the cached tag for the cached key id. Otherwise create your own
        """
        self.path = path or self.find_git()

    @property
    def revision(self):
        """
        Get current revision
        """
        revision = self.init_repo()
        cache.set(DJANGO_KIBRIT_CACHED_KEY_ID, revision, 60)
        return revision

    def init_repo(self):
        """
        The command to run and tag to return
        """
        try:
            return self.git('git describe --always --tags')
        except Exception, err:
            return ''

    def git(self, command=None, stderr=PIPE, stdout=PIPE, **kwargs):
        """
        Run a subprocess for the git command.
        """
        try:
            proc = Popen(
                command.split(), stderr=stderr, stdout=stdout,
                close_fds=(name == 'posix'), cwd=self.path, **kwargs)

        except Exception, err:
            pass
        try:
            output, error= [s.strip() for s in proc.communicate()]
        except Exception, err:
            output, error = '', ''
        return output

    def find_git(self, **kwargs):
        """
        Recursively finds a file directory named .git
        """
        command = "find -L ../src/ -type d -name .git".split()
        # TODO: can be made easier with __file__ and/or __name__
        command[2] = kwargs.get('path') or os.path.join(os.path.realpath(os.path.dirname(sys.argv[0])), '../src/')
        command[6] = kwargs.get('pattern') or command[6]
        try:
            proc = Popen(command, stderr=PIPE, stdout=PIPE,
                         close_fds=(name == 'posix'), cwd=os.path.dirname(command[2]), **kwargs)
        except Exception, err:
            pass
        try:
            output, error= [s.strip() for s in proc.communicate()]
        except Exception, err:
            output, error = '', ''
        return output



"""
Kibrit
------

SCM watcher tool. Get current revision and send update notify.
"""

import sys
from os import path as op

from setuptools import setup, find_packages

py_version = sys.version_info[:2]

PY3 = py_version[0] == 3

if PY3:
    if py_version < (3, 2):
        # raise RuntimeError('On Python 3, Kibrit requires Python 3.2 or better')
        raise RuntimeError('Havent Tested Kibrit on Python 3, \n'
                           '"Tread softly because you tread on my dreams."~W.B.Yeats')
else:
    if py_version < (2, 6):
        raise RuntimeError('On Python 2, Kibrit requires Python 2.6 or better')

def read(fname):
    try:
        return open(op.join(op.dirname(__file__), fname)).read()
    except IOError:
        return ''


NAME = 'django_kibrit'

CURDIR = op.dirname(__file__)
MODULE = __import__(NAME)
README = op.join(CURDIR, 'README.rst')
# REQUIREMENTS = open(op.join(CURDIR, 'requirements.txt')).readlines()
# if sys.version_info < (2, 7):
#     REQUIREMENTS.append('importlib')

setup(
    name=NAME,
    version=MODULE.__version__,
    license=MODULE.__license__,
    author=MODULE.__author__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    platforms=('Any'),
    keywords = "git static revision django".split(),

    maintainer='sinanm89',
    maintainer_email='sinanm89@gmail.com',
    author_email='sinanm89@gmail.com',
    url=' http://github.com/sinanm89/kibrit',
    download_url = 'https://github.com/sinanm89/kibrit/tarball/0.1',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Natural Language :: Turkish',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Framework :: Django",
        'Programming Language :: Python :: 2.6',
        'Environment :: Console',
    ],
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False
)

# list of all the classifiers https://pypi.python.org/pypi?%3Aaction=list_classifiers